(load "~/quicklisp/setup.lisp")

(eval-when (:compile-toplevel :load-toplevel :execute)
  (ql:quickload :cl-raylib))

(defpackage :raylib-user
  (:use :cl :3d-vectors)
  (:local-nicknames (:rl :raylib)))

(in-package :raylib-user)

(defparameter *move-speed* 2.0)

(defstruct game-state
  (textures (make-hash-table))
  (player-pos (vec 100 100)))

(defparameter *game-state* nil)

(defun bind-texture (key path)
  (setf (gethash key (game-state-textures *game-state*)) (rl:load-texture (uiop:native-namestring path))))

(defun game-init ()
  (setf *game-state* (make-game-state))
  (bind-texture 'terrain "~/Development/tinyswords/assets/Terrain/Ground/Tilemap_Flat.png")
  (bind-texture 'knight "~/Development/tinyswords/assets/Factions/Knights/Troops/Warrior/Blue/Warrior_Blue.png")
  (bind-texture 'tower "~/Development/tinyswords/assets/Factions/Knights/Buildings/Tower/Tower_Blue.png"))

(defun game-input (state)
  (with-slots ((pos player-pos)) state
    (let ((dx 0.0) (dy 0.0))
      (when (rl:is-key-down :key-right) (incf dx 1))
      (when (rl:is-key-down :key-left)  (decf dx 1))
      (when (rl:is-key-down :key-up)    (decf dy 1))
      (when (rl:is-key-down :key-down)  (incf dy 1))
      (setf pos (v+ pos (v* (vunit* (vec dx dy)) *move-speed*))))))

(defun game-update (state) '())

(defun get-tile-wrapped (n wrap-count) (if (> n 0) (1+ (mod (1- n) (- wrap-count 2))) 0))

(defun draw-tile (tilemap size col row)
  (let* ((tile-count 6))
    (rl:draw-texture-rec
     tilemap
     (rl:make-rectangle :x (* size (get-tile-wrapped col tile-count))
                        :y (* size (get-tile-wrapped row tile-count))
                        :width size :height size)
     (v+ (vec 15 150) (vec (* col size) (* row size)))
     :white)))

(defun draw-ground ()
  (let ((size 32)
        (terrain-tex (gethash 'terrain (game-state-textures *game-state*))))
    (loop for row from 0 to 16
          do (loop for col from 0 to 32
                   do (draw-tile terrain-tex size col row)))))

(defun animate-sprite ())

(defun game-draw (state)
  (rl:clear-background (rl:make-rgba 71 171 169 1))
  (draw-ground)
  (rl:draw-texture-v (gethash 'tower (game-state-textures state)) (vec 80 150) :white)
  (rl:draw-texture-rec (gethash 'knight (game-state-textures state))
                       (rl:make-rectangle :x 0.0 :y 0.0 :width 192.0 :height 192.0)
                       (vec 0 0)
                       :white)
  (rl:draw-fps 10 5))

(defun game ()
  (let* ((screen-width 900)
         (screen-height 500))
    (rl:with-window (screen-width screen-height "RTS")
      (rl:set-target-fps 60)
      (game-init)
      (loop :until (rl:window-should-close)
            :do (game-input *game-state*)
                (game-update *game-state*)
                (rl:with-drawing
                  (game-draw *game-state*)))
      (loop for value being the hash-values of (game-state-textures *game-state*)
            do (rl:unload-texture value)))))

(game)
